#  AST.py --- Class hierarchy supporting abstract syntax tree for parsed
#             formulas
#
#  last modified: 03/01/04 by HV
#
#
#                     =====================================
#                                 Marc Provost
#                          (marc.provost@mail.mcgill.ca)
#
#                                  Credits to:
#                             Jean-Sebastien BOLDUC
#                              (jseb@cs.mcgill.ca)
#
#                          McGill University, Montreal
#                     =====================================
#
#******************************************************************************

# This module defines convenient names for all built-in Python types (see
# Python documentation):
import types
from math import pow
from copy import copy
from ASTVisitor import ASTVisitor
from StrReprVisitor import StrReprVisitor

#******************************************************************************
# Utilities

#INPUT: <string>
#OUTPUT: <int>
#Order: O(length(string)) (in our case len(col) <= 3)
#Info: Function used to transform strings used
#      cell column references (e.g. A5) by integers
#      for efficiency reasons.  Note that 0 are removed
#      from possible integers to avoid ambiguity.
#      (i.e. if "A" is converted to 0, "AAA"=000=0,
#       0==0 while "A" != "AAA").
def stringToInt(col):
    base=ord("A")
    answer=0
    for l in xrange(len(col)):
        answer+=(ord(col[-(l+1)])-base+1)*int(pow(26,l))
    return answer

#INPUT: <string>
#OUTPUT: <int>
#Order: O(1) since integer assumed <= 26^3
#Info: Invert function of stringToInt()
def intToString(integer):
    ans=""
    base=ord("A")
    i=2
    for i in [2,1,0]:
        tmp=int(pow(26,i))
        tmpInteger=integer
        for j in xrange(i):
            tmpInteger-=int(pow(26,j))
        if tmp<=tmpInteger or i==0:
            charValue=(tmpInteger-tmpInteger%tmp)/tmp
            integer=integer-charValue*tmp
            char=chr(charValue+base-1)
            ans+=char
    return ans


